@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: client.c,v 1.42 2006/03/30 23:36:49 evertonm Exp $ */
+/* $Id: client.c,v 1.49 2006/04/19 10:35:03 evertonm Exp $ */
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -44,9 +44,10 @@ extern nepim_pipe_set_t pipes;    /* from server.c */
 extern nepim_slot_set_t slots;    /* from server.c */
 extern nepim_usock_set_t udp_tab; /* from server.c */
 
-static void *on_tcp_interval(oop_source *src, struct timeval tv, void *user);
 static void tcp_pipe_stop(int sd);
+static void *on_tcp_interval(oop_source *src, struct timeval tv, void *user);
 static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user);
+static void *on_tcp_keepalive_time(oop_source *src, struct timeval tv, void *user);
 
 static void schedule_stat_interval(int sd)
 {
@@ -96,9 +97,12 @@ static void *on_tcp_read(oop_source *src, int sd,
         fprintf(stderr, "read: errno=%d: %s\n",
                 errno, strerror(errno));
       }
-    }
 
-    fprintf(stderr, "read: connection lost on TCP socket %d\n", sd);
+      fprintf(stderr, "read: connection lost on TCP socket %d\n", sd);
+    }
+    else {
+      fprintf(stderr, "read: connection closed on TCP socket %d\n", sd);
+    }
 
     report_broken_pipe_stat(stdout, pipe);
 
@@ -113,6 +117,9 @@ static void *on_tcp_read(oop_source *src, int sd,
 
   session = &pipe->session;
 
+  /* any valid input traffic accounted as keepalive */
+  ++session->keepalives_recv;
+
   /* modifies 'session->seed' */
   tcp_check_data(sd, session, buf, rd);
 
@@ -398,28 +405,30 @@ static int greet(int sd)
   int wr;
   char *tmp = "";
 
-  opt.must_send             = nepim_global.duplex_mode ||
+  opt.must_send               = nepim_global.duplex_mode ||
     !nepim_global.simplex_client_send;
-  opt.bit_rate              = nepim_global.bit_rate;
-  opt.pkt_rate              = nepim_global.pkt_rate;
-  opt.stat_interval         = nepim_global.stat_interval;
-  opt.test_duration         = nepim_global.test_duration;
-  opt.write_delay           = nepim_global.write_delay;
-  opt.udp_keepalive_send    = -1;
-  opt.udp_keepalive_require = -1;
-  opt.seed                  = nepim_global.seed;
-  opt.verify_data           = nepim_global.verify_data;
-  opt.random_fill           = nepim_global.random_fill;
-  opt.fill_byte             = nepim_global.fill_byte;
-  opt.pause_duration        = nepim_global.pause_duration;
+  opt.bit_rate                = nepim_global.bit_rate;
+  opt.pkt_rate                = nepim_global.pkt_rate;
+  opt.stat_interval           = nepim_global.stat_interval;
+  opt.test_duration           = nepim_global.test_duration;
+  opt.write_delay             = nepim_global.write_delay;
+  opt.keepalive_must_send     = nepim_global.tcp_keepalive_require;
+  opt.keepalive_require       = nepim_global.tcp_keepalive_must_send;
+  opt.seed                    = nepim_global.seed;
+  opt.verify_data             = nepim_global.verify_data;
+  opt.random_fill             = nepim_global.random_fill;
+  opt.fill_byte               = nepim_global.fill_byte;
+  opt.pause_duration          = nepim_global.pause_duration;
+  opt.socket_keepalive        = nepim_global.socket_keepalive;
+  opt.nagle                   = nepim_global.nagle;
 
   if (nepim_global.password) {
-    opt.password_buf        = nepim_global.password;
-    opt.password_buf_size   = strlen(nepim_global.password) + 1;
+    opt.password_buf      = nepim_global.password;
+    opt.password_buf_size = strlen(nepim_global.password) + 1;
   }
   else {
-    opt.password_buf        = tmp;
-    opt.password_buf_size   = strlen(tmp) + 1;
+    opt.password_buf      = tmp;
+    opt.password_buf_size = strlen(tmp) + 1;
   }
 
   pr = nepim_write_greetings(&opt, buf, sizeof(buf));
@@ -436,6 +445,138 @@ static int greet(int sd)
   return 0;
 }
 
+static void *on_tcp_keepalive_write(oop_source *src, int sd,
+				    oop_event event, void *user)
+{
+  char buf[1];
+  int wr;
+
+  nepim_pipe_t *pipe = user;
+  nepim_session_t *session;
+
+  assert(event == OOP_WRITE);
+  assert(pipe);
+  assert(sd == pipe->sd);
+
+  session = &pipe->session;
+
+  assert(!session->must_send);
+  assert(session->keepalive_must_send);
+
+#if 0
+  fprintf(stderr, 
+	  "DEBUG %s %s: %d: sending keepalive\n",
+	  __FILE__, __PRETTY_FUNCTION__,
+	  sd);
+#endif
+
+  /* stop writing -- re-scheduled below */
+  nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
+                                  sd, OOP_WRITE);
+
+  wr = write(sd, buf, sizeof(buf));
+  if (wr != sizeof(buf)) {
+    switch (errno) {
+    case EINTR:
+      fprintf(stderr, "keepalive write: EINTR on TCP socket %d\n", sd);
+      return OOP_CONTINUE;
+    case EAGAIN:
+      fprintf(stderr, "keepalive write: EAGAIN on TCP socket %d\n", sd);
+      return OOP_CONTINUE;
+    case EPIPE:
+      fprintf(stderr, "keepalive write: EPIPE on TCP socket %d\n", sd);
+      break;
+    }
+    
+    fprintf(stderr, 
+	    "keepalive write: connection lost on TCP socket %d: errno=%d: %s\n", 
+	    sd, errno, strerror(errno));
+    
+    report_broken_pipe_stat(stdout, pipe);
+    
+    tcp_pipe_stop(sd);
+    close(sd);
+    
+    return OOP_CONTINUE;
+  }
+
+  assert(wr == sizeof(buf));
+
+  nepim_session_write_add(session, wr);
+
+  /* schedule next keepalive time */
+  nepim_global.oop_src->on_time(nepim_global.oop_src,
+                                session->tv_keepalive_send_next,
+                                on_tcp_keepalive_time, pipe);
+
+  return OOP_CONTINUE;
+}
+
+static void *on_tcp_keepalive_time(oop_source *src, struct timeval tv, void *user)
+{
+  nepim_pipe_t *pipe = user;
+  nepim_session_t *session;
+
+  assert(pipe);
+
+  session = &pipe->session;
+
+  assert(timercmp(&tv, &session->tv_keepalive_send_next, ==));
+  assert(!session->must_send);
+  assert(session->keepalive_must_send);
+
+  /* save next scheduling time */
+  {
+    int result = gettimeofday(&session->tv_keepalive_send_next, 0);
+    assert(!result);
+  }
+  nepim_timer_usec_add(&session->tv_keepalive_send_next,
+		       nepim_global.tcp_keepalive_send_delay);
+
+  /* start to write */
+  nepim_global.oop_src->on_fd(nepim_global.oop_src, pipe->sd, OOP_WRITE,
+			      on_tcp_keepalive_write, pipe);
+
+  return OOP_CONTINUE;
+}
+
+static void *on_tcp_keepalive_require(oop_source *src, struct timeval tv, void *user)
+{
+  nepim_pipe_t *pipe = user;
+  nepim_session_t *session = &pipe->session;
+
+  assert(session->keepalive_require);
+  assert(session->keepalives_recv >= 0);
+
+#if 0
+  fprintf(stderr, 
+	  "DEBUG %s %s: %d: keepalives_recv=%d\n",
+	  __FILE__, __PRETTY_FUNCTION__,
+	  pipe->sd, session->keepalives_recv);
+#endif
+
+  if (session->keepalives_recv < 1) {
+    int sd = pipe->sd;
+
+    fprintf(stderr, 
+            "%d: breaking pipe: peer not sending keepalives?\n",
+            sd);
+
+    report_broken_pipe_stat(stdout, pipe);
+
+    tcp_pipe_stop(sd);
+    close(sd);
+
+    return OOP_CONTINUE;
+  }
+
+  session->keepalives_recv = 0;
+
+  nepim_pipe_schedule_keepalive_timer(pipe, on_tcp_keepalive_require);
+
+  return OOP_CONTINUE;
+}
+
 static void tcp_pipe_start(int sd)
 {
   nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, sd);
@@ -461,6 +602,20 @@ static void tcp_pipe_start(int sd)
                                   on_tcp_write, pipe);
     }
   }
+  else {
+    if (session->keepalive_must_send) {
+
+      fprintf(stderr, 
+              "DEBUG %s %s: %d: scheduling sending of keepalives\n",
+              __FILE__, __PRETTY_FUNCTION__,
+              sd);
+      
+      session->tv_keepalive_send_next = OOP_TIME_NOW;
+      nepim_global.oop_src->on_time(nepim_global.oop_src,
+                                    session->tv_keepalive_send_next,
+                                    on_tcp_keepalive_time, pipe);
+    }
+  }
 
   {
     int result = gettimeofday(&session->tv_start, 0);
@@ -475,6 +630,9 @@ static void tcp_pipe_start(int sd)
   nepim_sock_show_opt(stderr, sd);
 
   schedule_stat_interval(sd);
+
+  if (session->keepalive_require)
+    nepim_pipe_schedule_keepalive_timer(pipe, on_tcp_keepalive_require);
 }
 
 static void *on_tcp_pipe_start(oop_source *src, struct timeval tv, void *user)
@@ -530,6 +688,20 @@ static void tcp_pipe_stop(int sd)
                                       sd, OOP_WRITE);
     }
   }
+  else {
+    if (session->keepalive_must_send) {
+
+      /* stop any will to write keepalives */
+      nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
+				      sd, OOP_WRITE);
+
+      /* stop periodic keepalive scheduler */
+      nepim_global.oop_src->cancel_time(nepim_global.oop_src,
+                                        session->tv_keepalive_send_next,
+                                        on_tcp_keepalive_time,
+                                        pipe);
+    }
+  }
 
   nepim_global.oop_src->cancel_time(nepim_global.oop_src,
                                     session->tv_duration,
@@ -539,6 +711,12 @@ static void tcp_pipe_stop(int sd)
                                     session->tv_interval,
                                     on_tcp_interval, pipe);
 
+  /* disable keepalive timer */
+  if (pipe->session.keepalive_require)
+    nepim_global.oop_src->cancel_time(nepim_global.oop_src,
+                                      pipe->session.tv_keepalive_recv_timer,
+                                      on_tcp_keepalive_require, pipe);
+
   nepim_pipe_set_del(&pipes, sd);
 
   nepim_sock_show_opt(stderr, sd);
@@ -553,6 +731,7 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
   struct stat stat_buf;
   int gai_lookup; /* boolean */
   int result;
+  int nodelay;
 
   memset(&hints, 0, sizeof(hints));
 
@@ -570,6 +749,9 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
       Lookup hostname
      */
 
+    nodelay = (nepim_global.nagle == -1) ?
+      -1 : !nepim_global.nagle;
+
     hints.ai_socktype = SOCK_STREAM;
     hints.ai_protocol = IPPROTO_TCP;
     hints.ai_flags = AI_CANONNAME;
@@ -596,6 +778,8 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
       Unix socket
      */
 
+    nodelay = -1; /* don't change */
+
     int len = strlen(hostname);
     if (len >= (int) sizeof(un_sock_addr.sun_path)) {
       fprintf(stderr, "%s %s: unix socket path overly long: len=%d > max=%d\n",
@@ -644,8 +828,9 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
                                      nepim_global.ttl,
                                      nepim_global.win_recv,
                                      nepim_global.win_send,
-				     gai_lookup /* nodelay */,
-				     nepim_global.tos);
+				     nodelay,
+				     nepim_global.tos,
+				     nepim_global.socket_keepalive);
     if (sd < 0) {
       if (ai->ai_family == AF_UNIX)
 	fprintf(stderr, 
@@ -674,20 +859,20 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
     {
       nepim_greet_t opt;
 
-      opt.must_send             = nepim_global.duplex_mode ||
+      opt.must_send               = nepim_global.duplex_mode ||
         nepim_global.simplex_client_send;
-      opt.bit_rate              = nepim_global.bit_rate;
-      opt.pkt_rate              = nepim_global.pkt_rate;
-      opt.stat_interval         = nepim_global.stat_interval;
-      opt.test_duration         = nepim_global.test_duration;
-      opt.write_delay           = nepim_global.write_delay;
-      opt.udp_keepalive_send    = nepim_global.udp_keepalive_send;
-      opt.udp_keepalive_require = nepim_global.udp_keepalive_require;
-      opt.seed                  = nepim_global.seed;
-      opt.verify_data           = nepim_global.verify_data;
-      opt.random_fill           = nepim_global.random_fill;
-      opt.fill_byte             = nepim_global.fill_byte;
-      opt.pause_duration        = nepim_global.pause_duration;
+      opt.bit_rate                = nepim_global.bit_rate;
+      opt.pkt_rate                = nepim_global.pkt_rate;
+      opt.stat_interval           = nepim_global.stat_interval;
+      opt.test_duration           = nepim_global.test_duration;
+      opt.write_delay             = nepim_global.write_delay;
+      opt.keepalive_must_send     = nepim_global.tcp_keepalive_require;
+      opt.keepalive_require       = nepim_global.tcp_keepalive_must_send;
+      opt.seed                    = nepim_global.seed;
+      opt.verify_data             = nepim_global.verify_data;
+      opt.random_fill             = nepim_global.random_fill;
+      opt.fill_byte               = nepim_global.fill_byte;
+      opt.pause_duration          = nepim_global.pause_duration;
 
       nepim_pipe_set_add(&pipes, sd, ai->ai_addr, ai->ai_addrlen, &opt);
     }
