@@ -57,10 +57,10 @@ struct nepim_global_t {
   int           mcast_ttl;
   susec_t       udp_greet_delay;         /* usec */
   int           max_greetings;
-  susec_t       udp_keepalive_timer;     /* usec, receivers */
-  susec_t       udp_keepalive_delay;     /* usec, non-senders */
-  int           udp_keepalive_require;   /* boolean, receivers */
-  int           udp_keepalive_send;      /* boolean, non-senders */
+  susec_t       udp_keepalive_recv_timer; /* usec, receivers */
+  susec_t       udp_keepalive_send_delay; /* usec, non-senders */
+  int           udp_keepalive_require;    /* boolean, receivers */
+  int           udp_keepalive_must_send;  /* boolean, non-senders */
   int           udp_win_max;
   int           win_recv;
   int           win_send;
@@ -78,6 +78,12 @@ struct nepim_global_t {
   int           search_unix_socket; /* boolean */
   int           soft_error_verbose; /* boolean */
   int           tos;
+  int           socket_keepalive; /* client, boolean, -1 is don't change */
+  susec_t       tcp_keepalive_recv_timer; /* usec, receivers */
+  susec_t       tcp_keepalive_send_delay; /* usec, non-senders */
+  int           tcp_keepalive_require;    /* boolean, receivers */
+  int           tcp_keepalive_must_send;  /* boolean, non-senders */
+  int           nagle; /* client, boolean, -1 is don't change */
 
   oop_source_sys *oop_sys;
   oop_source     *oop_src;
