@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: greet.h,v 1.8 2005/12/20 09:36:09 evertonm Exp $ */
+/* $Id: greet.h,v 1.12 2006/04/18 22:56:33 evertonm Exp $ */
 
 #ifndef NEPIM_GREET_H
 #define NEPIM_GREET_H
@@ -32,13 +32,15 @@ struct nepim_greet_t {
   int           stat_interval;
   int           test_duration;
   long          write_delay;
-  int           udp_keepalive_send;
-  int           udp_keepalive_require;
+  int           keepalive_must_send;
+  int           keepalive_require;
   unsigned      seed;
   unsigned      verify_data;
   int           random_fill;
   unsigned char fill_byte;
   int           pause_duration;
+  int           socket_keepalive;
+  int           nagle;
   char          *password_buf;
   int           password_buf_size;
 };
