@@ -21,7 +21,7 @@ MA 02110-1301 USA
 /*
   nepim - network pipemeter
 
-  $Id: main.c,v 1.40 2006/04/07 21:46:21 evertonm Exp $
+  $Id: main.c,v 1.45 2006/04/19 12:02:07 evertonm Exp $
  */
 
 #include <stdlib.h>
@@ -120,6 +120,15 @@ static void usage(FILE *out, const char *prog_name)
           "  -X pause       wait \"pause\" seconds before transmitting data\n"
           "                 (defaults to %d seconds)\n"
           "  -g retries     limit for UDP greeting retries (defaults to %d)\n"
+	  "  -A socket-ka   socket keepalive (defaults to %d)\n"
+	  "                 socket-ka: -1 = use system's default\n"
+	  "                             0 = off\n"
+	  "                             1 = on\n"
+	  "  -C             enable app-layer TCP keepalives (defaults to off)\n"
+	  "  -N nagle       TCP nagle mode (defaults to %d)\n"
+	  "                 nagle: -1 = use system's default\n"
+	  "                         0 = off (enable TCP_NODELAY)\n"
+	  "                         1 = on (disable TCP_NODELAY)\n"
           "server-only options:\n"
           "  -f [lib-name]  enable TCP wrapper filtering\n"
           "  -b addr-list   bind to specific addresses (defaults to any)\n"
@@ -142,7 +151,7 @@ static void usage(FILE *out, const char *prog_name)
           nepim_global.udp_write_size,
           nepim_global.udp_read_size,
 	  nepim_global.tos,
-	  (long) nepim_global.udp_keepalive_timer,
+	  (long) nepim_global.udp_keepalive_recv_timer,
           nepim_global.udp_win_max,
           nepim_global.pipes,
           bit_rate,
@@ -151,6 +160,8 @@ static void usage(FILE *out, const char *prog_name)
           nepim_global.test_duration,
           nepim_global.pause_duration,
           nepim_global.max_greetings,
+          nepim_global.socket_keepalive,
+          nepim_global.nagle,
           nepim_global.listen_backlog
           );
 }
@@ -167,16 +178,18 @@ static void init_event_loop()
   assert(nepim_global.oop_src);
 }
 
-static void show_version_brief() 
+static void show_version_brief(FILE *out) 
 {
-  printf("nepim - network pipemeter - version %s\n", nepim_version());
+  fprintf(out,
+	  "nepim - network pipemeter - version %s\n",
+	  nepim_version());
 }
 
-static void show_version() 
+static void show_version(FILE *out) 
 {
-  show_version_brief();
+  show_version_brief(out);
 
-  printf(
+  fprintf(out,
          "Copyright (C) 2005 Everton da Silva marques\n"
 "\n"
 "This program is free software; you can redistribute it and/or\n"
@@ -203,6 +216,34 @@ int main(int argc, const char *argv[])
   for (i = 1; i < argc; ++i) {
     const char *arg = argv[i];
 
+    if (!strcmp(arg, "-N")) {
+      ++i;
+      if (i >= argc) {
+        fprintf(stderr, "%s: missing nagle mode\n",
+                nepim_global.prog_name);
+        exit(1);
+      }
+      nepim_global.nagle = atoi(argv[i]);
+      continue;
+    }
+
+    if (!strcmp(arg, "-C")) {
+      nepim_global.tcp_keepalive_require = 1; /* true */
+      nepim_global.tcp_keepalive_must_send = 1; /* true */
+      continue;
+    }
+
+    if (!strcmp(arg, "-A")) {
+      ++i;
+      if (i >= argc) {
+        fprintf(stderr, "%s: missing socket keepalive mode\n",
+                nepim_global.prog_name);
+        exit(1);
+      }
+      nepim_global.socket_keepalive = atoi(argv[i]);
+      continue;
+    }
+
     if (!strcmp(arg, "-y")) {
       ++i;
       if (i >= argc) {
@@ -221,7 +262,7 @@ int main(int argc, const char *argv[])
                 nepim_global.prog_name);
         exit(1);
       }
-      nepim_global.udp_keepalive_timer = atoi(argv[i]);
+      nepim_global.udp_keepalive_recv_timer = atoi(argv[i]);
       continue;
     }
 
@@ -581,7 +622,7 @@ int main(int argc, const char *argv[])
     }
 
     if (!strcmp(arg, "-v")) {
-      show_version();
+      show_version(stdout);
       exit(0);
     }
 
@@ -675,7 +716,7 @@ int main(int argc, const char *argv[])
     exit(1);
   }
 
-  show_version_brief();
+  show_version_brief(stderr);
 
   init_event_loop();
 
