@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: server.c,v 1.50 2006/03/31 18:52:52 evertonm Exp $ */
+/* $Id: server.c,v 1.60 2006/04/19 10:35:03 evertonm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -47,14 +47,15 @@ nepim_pipe_set_t  pipes;   /* tcp socket table */
 nepim_slot_set_t  slots;   /* udp pseudo-sockets */
 nepim_usock_set_t udp_tab; /* udp socket table */
 
-const char *INET_ANY = "0.0.0.0";
-const char *INET6_ANY = "::";
+static const char * const INET_ANY = "0.0.0.0";
+static const char * const INET6_ANY = "::";
 
 static void schedule_stat_interval(int sd);
 static void tcp_pipe_cancel_timers(int sd);
 static void tcp_pipe_cancel_io(int sd);
 static void tcp_pipe_kill(int sd);
 static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user);
+static void *on_tcp_keepalive_time(oop_source *src, struct timeval tv, void *user);
 
 static void *on_tcp_read(oop_source *src, int sd,
                          oop_event event, void *user)
@@ -82,9 +83,12 @@ static void *on_tcp_read(oop_source *src, int sd,
         fprintf(stderr, "read: errno=%d: %s\n",
                 errno, strerror(errno));
       }
-    }
 
-    fprintf(stderr, "read: connection lost on TCP socket %d\n", sd);
+      fprintf(stderr, "read: connection lost on TCP socket %d\n", sd);
+    }
+    else {
+      fprintf(stderr, "read: connection closed on TCP socket %d\n", sd);
+    }
 
     if (!session->duration_done)
       report_broken_pipe_stat(stdout, pipe);
@@ -96,6 +100,9 @@ static void *on_tcp_read(oop_source *src, int sd,
 
   assert(rd >= 0);
   assert(rd <= sizeof(buf));
+
+  /* any valid input traffic accounted as keepalive */
+  ++session->keepalives_recv;
   
   /* modifies 'session->seed' */
   tcp_check_data(sd, session, buf, rd);
@@ -371,6 +378,7 @@ static void *on_tcp_interval(oop_source *src, struct timeval tv, void *user)
 }
 
 static int parse_greetings(int sd,
+			   int sockaddr_family,
                            const struct sockaddr *remote,
                            socklen_t remote_len,
                            const char *buf, const char *past_end)
@@ -378,6 +386,7 @@ static int parse_greetings(int sd,
   nepim_greet_t opt;
   int result;
   char password_buf[100];
+  int nodelay;
 
   opt.password_buf = password_buf;
   opt.password_buf_size = sizeof(password_buf);
@@ -390,12 +399,13 @@ static int parse_greetings(int sd,
           "%d: send=%d bit_rate=%lld pkt_rate=%d "
           "interval=%d duration=%d delay=%ld ka_send=%d ka_req=%d seed=0x%08x "
           "verify_data=%d random_fill=%d fill_byte=0x%02x pause_duration=%d "
-          "password=%s\n", 
+          "sock_ka=%d nagle=%d password=%s\n", 
           sd, opt.must_send, opt.bit_rate, opt.pkt_rate,
           opt.stat_interval, opt.test_duration, 
-          opt.write_delay, opt.udp_keepalive_send,
-          opt.udp_keepalive_require, opt.seed,
+          opt.write_delay, opt.keepalive_must_send,
+          opt.keepalive_require, opt.seed,
           opt.verify_data, opt.random_fill, opt.fill_byte, opt.pause_duration,
+	  opt.socket_keepalive, opt.nagle,
           password_buf);
 
   if (nepim_global.password)
@@ -405,12 +415,29 @@ static int parse_greetings(int sd,
       return 1;
     }
 
+  if (nepim_socket_keepalive(sd, opt.socket_keepalive))
+    return 1;
+
+  if (sockaddr_family == PF_UNIX)
+    nodelay = -1; /* dont change */
+  else
+    nodelay = (opt.nagle == -1) ? -1 : !opt.nagle;
+
+  result = nepim_socket_tcp_opt(sd, nodelay);
+  if (result) {
+    fprintf(stderr, 
+            "%d: %s: %s: could not set tcp options: %d\n",
+            sd, __FILE__, __PRETTY_FUNCTION__, result);
+    return 1;
+  }
+
   nepim_pipe_set_add(&pipes, sd, remote, remote_len, &opt);
 
   return 0;
 }
 
 static int read_greetings(int sd,
+			  int sockaddr_family,
                           const struct sockaddr *remote,
                           socklen_t remote_len)
 {
@@ -432,6 +459,8 @@ static int read_greetings(int sd,
       return -1;
     }
     if (rd < 0) {
+      if (errno == EINTR)
+        continue;
       if (errno == EAGAIN)
         continue;
     }
@@ -447,7 +476,8 @@ static int read_greetings(int sd,
 
   *eos = '\0';
 
-  result = parse_greetings(sd, remote, remote_len, buf, eos);
+  result = parse_greetings(sd, sockaddr_family,
+			   remote, remote_len, buf, eos);
   if (result) {
     fprintf(stderr, 
             "%d: bad client greetings: %d [%s]\n", 
@@ -478,6 +508,138 @@ static void schedule_stat_interval(int sd)
                                 on_tcp_interval, pipe);
 }
 
+static void *on_tcp_keepalive_write(oop_source *src, int sd,
+				    oop_event event, void *user)
+{
+  char buf[1];
+  int wr;
+
+  nepim_pipe_t *pipe = user;
+  nepim_session_t *session;
+
+  assert(event == OOP_WRITE);
+  assert(pipe);
+  assert(sd == pipe->sd);
+
+  session = &pipe->session;
+
+  assert(!session->must_send);
+  assert(session->keepalive_must_send);
+
+#if 0
+  fprintf(stderr, 
+	  "DEBUG %s %s: %d: sending keepalive\n",
+	  __FILE__, __PRETTY_FUNCTION__,
+	  sd);
+#endif
+
+  /* stop writing -- re-scheduled below */
+  nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
+                                  sd, OOP_WRITE);
+
+  wr = write(sd, buf, sizeof(buf));
+  if (wr != sizeof(buf)) {
+    switch (errno) {
+    case EINTR:
+      fprintf(stderr, "keepalive write: EINTR on TCP socket %d\n", sd);
+      return OOP_CONTINUE;
+    case EAGAIN:
+      fprintf(stderr, "keepalive write: EAGAIN on TCP socket %d\n", sd);
+      return OOP_CONTINUE;
+    case EPIPE:
+      fprintf(stderr, "keepalive write: EPIPE on TCP socket %d\n", sd);
+      break;
+    }
+    
+    fprintf(stderr, 
+	    "keepalive write: connection lost on TCP socket %d: errno=%d: %s\n", 
+	    sd, errno, strerror(errno));
+    
+    if (!session->duration_done)
+      report_broken_pipe_stat(stdout, pipe);
+    
+    tcp_pipe_kill(sd);
+    
+    return OOP_CONTINUE;
+  }
+
+  assert(wr == sizeof(buf));
+
+  nepim_session_write_add(session, wr);
+
+  /* schedule next keepalive time */
+  nepim_global.oop_src->on_time(nepim_global.oop_src,
+                                session->tv_keepalive_send_next,
+                                on_tcp_keepalive_time, pipe);
+
+  return OOP_CONTINUE;
+}
+
+static void *on_tcp_keepalive_time(oop_source *src, struct timeval tv, void *user)
+{
+  nepim_pipe_t *pipe = user;
+  nepim_session_t *session;
+
+  assert(pipe);
+
+  session = &pipe->session;
+
+  assert(timercmp(&tv, &session->tv_keepalive_send_next, ==));
+  assert(!session->must_send);
+  assert(session->keepalive_must_send);
+
+  /* save next scheduling time */
+  {
+    int result = gettimeofday(&session->tv_keepalive_send_next, 0);
+    assert(!result);
+  }
+  nepim_timer_usec_add(&session->tv_keepalive_send_next,
+		       nepim_global.tcp_keepalive_send_delay);
+
+  /* start to write */
+  nepim_global.oop_src->on_fd(nepim_global.oop_src, pipe->sd, OOP_WRITE,
+			      on_tcp_keepalive_write, pipe);
+
+  return OOP_CONTINUE;
+}
+
+static void *on_tcp_keepalive_require(oop_source *src, struct timeval tv, void *user)
+{
+  nepim_pipe_t *pipe = user;
+  nepim_session_t *session = &pipe->session;
+
+  assert(session->keepalive_require);
+  assert(session->keepalives_recv >= 0);
+
+#if 0
+  fprintf(stderr, 
+	  "DEBUG %s %s: %d: keepalives_recv=%d\n",
+	  __FILE__, __PRETTY_FUNCTION__,
+	  pipe->sd, session->keepalives_recv);
+#endif
+
+  if (session->keepalives_recv < 1) {
+    int sd = pipe->sd;
+
+    fprintf(stderr, 
+            "%d: breaking pipe: peer not sending keepalives?\n",
+            sd);
+
+    if (!session->duration_done)
+      report_broken_pipe_stat(stdout, pipe);
+
+    tcp_pipe_kill(sd);
+
+    return OOP_CONTINUE;
+  }
+
+  session->keepalives_recv = 0;
+
+  nepim_pipe_schedule_keepalive_timer(pipe, on_tcp_keepalive_require);
+
+  return OOP_CONTINUE;
+}
+
 static void tcp_pipe_start(int sd)
 {
   nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, sd);
@@ -503,6 +665,20 @@ static void tcp_pipe_start(int sd)
                                   on_tcp_write, pipe);
     }
   }
+  else {
+    if (session->keepalive_must_send) {
+
+      fprintf(stderr, 
+              "DEBUG %s %s: %d: scheduling sending of keepalives\n",
+              __FILE__, __PRETTY_FUNCTION__,
+              sd);
+      
+      session->tv_keepalive_send_next = OOP_TIME_NOW;
+      nepim_global.oop_src->on_time(nepim_global.oop_src,
+                                    session->tv_keepalive_send_next,
+                                    on_tcp_keepalive_time, pipe);
+    }
+  }
 
   {
     int result = gettimeofday(&session->tv_start, 0);
@@ -517,6 +693,9 @@ static void tcp_pipe_start(int sd)
   nepim_sock_show_opt(stderr, sd);
 
   schedule_stat_interval(sd);
+
+  if (session->keepalive_require)
+    nepim_pipe_schedule_keepalive_timer(pipe, on_tcp_keepalive_require);
 }
 
 static void *on_tcp_pipe_start(oop_source *src, struct timeval tv, void *user)
@@ -560,6 +739,12 @@ static void tcp_pipe_cancel_timers(int sd)
   nepim_global.oop_src->cancel_time(nepim_global.oop_src,
                                     session->tv_interval,
                                     on_tcp_interval, pipe);
+
+  /* disable keepalive timer */
+  if (pipe->session.keepalive_require)
+    nepim_global.oop_src->cancel_time(nepim_global.oop_src,
+                                      pipe->session.tv_keepalive_recv_timer,
+                                      on_tcp_keepalive_require, pipe);
 }
 
 static void tcp_pipe_cancel_io(int sd)
@@ -589,11 +774,37 @@ static void tcp_pipe_cancel_io(int sd)
                                       sd, OOP_WRITE);
     }
   }
+  else {
+    assert(!session->must_send);
+
+    if (session->keepalive_must_send) {
+      
+      /* stop any will to write keepalives */
+      nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
+				      sd, OOP_WRITE);
+
+      /* stop periodic keepalive scheduler */
+      nepim_global.oop_src->cancel_time(nepim_global.oop_src,
+                                        session->tv_keepalive_send_next,
+                                        on_tcp_keepalive_time,
+                                        pipe);
+    }
+  }
+
 }
 
 static void tcp_pipe_kill(int sd)
 {
-  nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, sd);
+  nepim_pipe_t *pipe;
+
+#if 0
+  fprintf(stderr, 
+	  "DEBUG %s %s: %d: closing sd\n",
+	  __FILE__, __PRETTY_FUNCTION__,
+	  sd);
+#endif
+
+  pipe = nepim_pipe_set_get(&pipes, sd);
   assert(pipe);
 
   tcp_pipe_cancel_timers(sd);
@@ -668,15 +879,6 @@ static void *on_tcp_connect(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  result = nepim_socket_tcp_opt(conn_sd, family != PF_UNIX /* nodelay */);
-  if (result) {
-    fprintf(stderr, 
-            "%d: %s: %s: could not set tcp options: %d\n",
-            conn_sd, __FILE__, __PRETTY_FUNCTION__, result);
-    close(conn_sd);
-    return OOP_CONTINUE;
-  }
-
   if (nepim_socket_block(conn_sd)) {
     fprintf(stderr, 
             "%d: %s: %s: could not set blocking socket mode\n",
@@ -685,7 +887,8 @@ static void *on_tcp_connect(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  if (read_greetings(conn_sd, (const struct sockaddr *) &sa, len)) {
+  if (read_greetings(conn_sd, family,
+		     (const struct sockaddr *) &sa, len)) {
     fprintf(stderr, 
             "%d: %s: %s: could not parse client greetings\n",
             conn_sd, __FILE__, __PRETTY_FUNCTION__);
