@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: sock.c,v 1.30 2006/03/31 18:52:53 evertonm Exp $ */
+/* $Id: sock.c,v 1.33 2006/04/18 23:14:19 evertonm Exp $ */
 
 #include <assert.h>
 #include <netdb.h>
@@ -89,6 +89,7 @@ struct ipv6_mreq {
 #define NEPIM_SOCK_ERR_WIN_RECV       (-15)
 #define NEPIM_SOCK_ERR_WIN_SEND       (-16)
 #define NEPIM_SOCK_ERR_TOS            (-17)
+#define NEPIM_SOCK_ERR_SOCK_KA        (-18)
 
 int nepim_sock_get_port(const struct sockaddr *addr)
 {
@@ -476,11 +477,44 @@ static int reuse(int sd)
   return setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));
 }
 
-static int tcp_nodelay(int sd)
+static int set_tcp_nodelay(int sd, int nodelay)
 {
-  int opt = 1;
+  if (nodelay == -1)
+    return NEPIM_SOCK_ERR_NONE;
 
-  return setsockopt(sd, SOL_TCP, TCP_NODELAY, &opt, sizeof(opt));
+  if (setsockopt(sd, SOL_TCP, TCP_NODELAY,
+		 &nodelay, sizeof(nodelay))) {
+    int e = errno;
+    fprintf(stderr, "%s: %s: set socket TCP_NODELAY: sd=%d: errno=%d: %s\n",
+	    __FILE__, __PRETTY_FUNCTION__,
+	    sd, e, strerror(e));
+
+    return NEPIM_SOCK_ERR_NODELAY;
+  }
+
+  return NEPIM_SOCK_ERR_NONE;
+}
+
+static int nepim_socket_get_nodelay(int sd)
+{
+  int nodelay;
+  socklen_t optlen = sizeof(nodelay);
+
+  if (getsockopt(sd, SOL_TCP, TCP_NODELAY,
+		 &nodelay, &optlen)) {
+#if 0
+    int e = errno;
+    fprintf(stderr, "%s: %s: get socket TCP_NODELAY: sd=%d: errno=%d: %s\n",
+	    __FILE__, __PRETTY_FUNCTION__,
+	    sd, e, strerror(e));
+#endif
+
+    return NEPIM_SOCK_ERR_NODELAY;
+  }
+  
+  assert(optlen == sizeof(nodelay));
+  
+  return nodelay;
 }
 
 int nepim_socket_opt(int sd, int pmtu_mode, int ttl, int tos)
@@ -505,13 +539,48 @@ int nepim_socket_opt(int sd, int pmtu_mode, int ttl, int tos)
 
 int nepim_socket_tcp_opt(int sd, int nodelay)
 {
-  if (nodelay)
-    if (tcp_nodelay(sd))
-      return NEPIM_SOCK_ERR_NODELAY;
+  if (set_tcp_nodelay(sd, nodelay))
+    return NEPIM_SOCK_ERR_NODELAY;
 
   return NEPIM_SOCK_ERR_NONE;
 }
 
+int nepim_socket_keepalive(int sd, int keepalive)
+{
+  if (keepalive == -1)
+    return NEPIM_SOCK_ERR_NONE;
+
+  if (setsockopt(sd, SOL_SOCKET, SO_KEEPALIVE,
+		 &keepalive, sizeof(keepalive))) {
+    int e = errno;
+    fprintf(stderr, "%s: %s: set socket keepalive: sd=%d: errno=%d: %s\n",
+	    __FILE__, __PRETTY_FUNCTION__,
+	    sd, e, strerror(e));
+    return NEPIM_SOCK_ERR_SOCK_KA;
+  }
+
+  return NEPIM_SOCK_ERR_NONE;
+}
+
+static int nepim_socket_get_keepalive(int sd)
+{
+  int keepalive;
+  socklen_t optlen = sizeof(keepalive);
+
+  if (getsockopt(sd, SOL_SOCKET, SO_KEEPALIVE,
+		 &keepalive, &optlen)) {
+    int e = errno;
+    fprintf(stderr, "%s: %s: get socket keepalive: sd=%d: errno=%d: %s\n",
+	    __FILE__, __PRETTY_FUNCTION__,
+	    sd, e, strerror(e));
+    return NEPIM_SOCK_ERR_SOCK_KA;
+  }
+  
+  assert(optlen == sizeof(keepalive));
+  
+  return keepalive;
+}
+
 int nepim_connect_client_socket(struct sockaddr *addr,
 				int addr_len,
 				int family,
@@ -522,7 +591,8 @@ int nepim_connect_client_socket(struct sockaddr *addr,
 				int win_recv,
 				int win_send,
 				int nodelay,
-				int tos)
+				int tos,
+				int socket_keepalive)
 {
   int sd;
   int result;
@@ -543,6 +613,12 @@ int nepim_connect_client_socket(struct sockaddr *addr,
  }
 #endif /* Linux SO_BSDCOMPAT */
 
+  result = nepim_socket_keepalive(sd, socket_keepalive);
+  if (result) {
+    close(sd);
+    return result;
+  }
+
   result = nepim_socket_block(sd);
   if (result) {
     close(sd);
@@ -690,6 +766,8 @@ void nepim_sock_show_opt(FILE *out, int sd)
   int win_recv;
   int win_send;
   int tos;
+  int keepalive;
+  int nodelay;
 
   pmtud_mode = nepim_socket_pmtu_get_mode(sd);
   mtu = nepim_socket_pmtu_get_mtu(sd);
@@ -698,8 +776,17 @@ void nepim_sock_show_opt(FILE *out, int sd)
   win_recv = socket_get_win_recv(sd);
   win_send = socket_get_win_send(sd);
   tos = nepim_socket_get_tos(sd);
+  keepalive = nepim_socket_get_keepalive(sd);
+  nodelay = nepim_socket_get_nodelay(sd);
 
   fprintf(out, 
-	  "%d: pmtud_mode=%d path_mtu=%d tos=%d ttl=%d mcast_ttl=%d win_recv=%d win_send=%d\n",
-	  sd, pmtud_mode, mtu, tos, ttl, mcast_ttl, win_recv, win_send);
+	  "%d:"
+	  " pmtud_mode=%d path_mtu=%d tos=%d ttl=%d"
+	  " mcast_ttl=%d win_recv=%d win_send=%d sock_ka=%d"
+	  " nodelay=%d\n"
+	  ,
+	  sd,
+	  pmtud_mode, mtu, tos, ttl,
+	  mcast_ttl, win_recv, win_send, keepalive,
+	  nodelay);
 }
