@@ -31,6 +31,7 @@ int nepim_socket_mcast_ttl(int sd, int mcast_ttl);
 int nepim_socket_pmtu(int sd, int pmtu_mode);
 int nepim_socket_opt(int sd, int pmtu_mode, int ttl, int tos);
 int nepim_socket_tcp_opt(int sd, int nodelay);
+int nepim_socket_keepalive(int sd, int keepalive);
 
 int nepim_socket_pmtu_get_mode(int sd);
 int nepim_socket_pmtu_get_mtu(int sd);
@@ -71,7 +72,8 @@ int nepim_connect_client_socket(struct sockaddr *addr,
 				int win_recv,
 				int win_send,
 				int nodelay,
-				int tos);
+				int tos,
+				int sock_keepalive);
 
 int nepim_sock_get_port(const struct sockaddr *addr);
 void nepim_sock_dump_addr(char *buf, int buf_size,
