@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: udp_client.c,v 1.51 2006/03/30 23:36:49 evertonm Exp $ */
+/* $Id: udp_client.c,v 1.56 2006/04/18 12:10:15 evertonm Exp $ */
 
 
 #include <stdio.h>
@@ -82,10 +82,10 @@ static void *on_udp_keepalive_require(oop_source *src, struct timeval tv, void *
   nepim_slot_t *slot = user;
   nepim_session_t *session = &slot->session;
 
-  assert(session->udp_keepalive_require);
-  assert(slot->keepalives_recv >= 0);
+  assert(session->keepalive_require);
+  assert(session->keepalives_recv >= 0);
 
-  if (slot->keepalives_recv < 1) {
+  if (session->keepalives_recv < 1) {
     fprintf(stderr, 
             "%d %d-%d: broken slot: peer not sending keepalives?\n",
             slot->udp_sd, slot->index, slot->index_remote);
@@ -97,7 +97,7 @@ static void *on_udp_keepalive_require(oop_source *src, struct timeval tv, void *
     return OOP_CONTINUE;
   }
 
-  slot->keepalives_recv = 0;
+  session->keepalives_recv = 0;
 
   nepim_schedule_keepalive_timer(slot, on_udp_keepalive_require);
 
@@ -109,16 +109,17 @@ static void *on_udp_keepalive_time(oop_source *src, struct timeval tv, void *use
   nepim_slot_t *slot = user;
   nepim_session_t *session = &slot->session;
 
-  assert(timercmp(&tv, &session->tv_keepalive, ==));
+  assert(timercmp(&tv, &session->tv_keepalive_send_next, ==));
   assert(!session->must_send);
-  assert(session->udp_keepalive_send);
+  assert(session->keepalive_must_send);
 
   /* save next scheduling time */
   {
-    int result = gettimeofday(&session->tv_keepalive, 0);
+    int result = gettimeofday(&session->tv_keepalive_send_next, 0);
     assert(!result);
   }
-  nepim_timer_usec_add(&session->tv_keepalive, nepim_global.udp_keepalive_delay);
+  nepim_timer_usec_add(&session->tv_keepalive_send_next,
+		       nepim_global.udp_keepalive_send_delay);
 
   /* start to write */
   nepim_will_slot_keepalive(slot, on_udp_write);
@@ -144,16 +145,16 @@ static void udp_slot_start(nepim_slot_t *slot)
     }
   }
   else {
-    if (session->udp_keepalive_send) {
+    if (session->keepalive_must_send) {
 
       fprintf(stderr, 
               "DEBUG %s %s: %d %d-%d: scheduling sending of keepalives\n",
               __FILE__, __PRETTY_FUNCTION__,
               slot->udp_sd, slot->index, slot->index_remote);
       
-      session->tv_keepalive = OOP_TIME_NOW;
+      session->tv_keepalive_send_next = OOP_TIME_NOW;
       nepim_global.oop_src->on_time(nepim_global.oop_src,
-                                    session->tv_keepalive,
+                                    session->tv_keepalive_send_next,
                                     on_udp_keepalive_time, slot);
     }
   }
@@ -172,7 +173,7 @@ static void udp_slot_start(nepim_slot_t *slot)
 
   schedule_stat_interval(slot);
 
-  if (session->udp_keepalive_require)
+  if (session->keepalive_require)
     nepim_schedule_keepalive_timer(slot, on_udp_keepalive_require);
 }
 
@@ -193,8 +194,8 @@ static void schedule_udp_slot_start(nepim_slot_t *slot)
   session = &slot->session;
 
   {
-      int result = gettimeofday(&pause, 0);
-      assert(!result);
+    int result = gettimeofday(&pause, 0);
+    assert(!result);
   }
   pause.tv_sec += session->pause_duration;
   nepim_global.oop_src->on_time(nepim_global.oop_src,
@@ -206,8 +207,8 @@ static void udp_consume(nepim_slot_t *slot, size_t len, uint32_t seq)
 {
   nepim_session_t *session = &slot->session;
 
-  /* any valid traffic accounted as keepalive */
-  ++slot->keepalives_recv;
+  /* any valid input traffic accounted as keepalive */
+  ++session->keepalives_recv;
 
   nepim_session_read_add(session, len);
 
@@ -347,28 +348,29 @@ static int slot_greet_write(nepim_slot_t *slot)
 
   assert(sizeof(buf) > UDP_HEADER_LEN);
 
-  opt.must_send             = nepim_global.duplex_mode ||
+  opt.must_send               = nepim_global.duplex_mode ||
     !nepim_global.simplex_client_send;
-  opt.bit_rate              = nepim_global.bit_rate;
-  opt.pkt_rate              = nepim_global.pkt_rate;
-  opt.stat_interval         = nepim_global.stat_interval;
-  opt.test_duration         = nepim_global.test_duration;
-  opt.write_delay           = nepim_global.write_delay;
-  opt.udp_keepalive_send    = nepim_global.udp_keepalive_require;
-  opt.udp_keepalive_require = nepim_global.udp_keepalive_send;
-  opt.seed                  = nepim_global.seed;
-  opt.verify_data           = nepim_global.verify_data;
-  opt.random_fill           = nepim_global.random_fill;
-  opt.fill_byte             = nepim_global.fill_byte;
-  opt.pause_duration        = nepim_global.pause_duration;
+  opt.bit_rate                = nepim_global.bit_rate;
+  opt.pkt_rate                = nepim_global.pkt_rate;
+  opt.stat_interval           = nepim_global.stat_interval;
+  opt.test_duration           = nepim_global.test_duration;
+  opt.write_delay             = nepim_global.write_delay;
+  opt.keepalive_must_send     = nepim_global.udp_keepalive_require;
+  opt.keepalive_require       = nepim_global.udp_keepalive_must_send;
+  opt.seed                    = nepim_global.seed;
+  opt.verify_data             = nepim_global.verify_data;
+  opt.random_fill             = nepim_global.random_fill;
+  opt.fill_byte               = nepim_global.fill_byte;
+  opt.pause_duration          = nepim_global.pause_duration;
+  opt.socket_keepalive        = nepim_global.socket_keepalive;
 
   if (nepim_global.password) {
-    opt.password_buf        = nepim_global.password;
-    opt.password_buf_size   = strlen(nepim_global.password) + 1;
+    opt.password_buf      = nepim_global.password;
+    opt.password_buf_size = strlen(nepim_global.password) + 1;
   }
   else {
-    opt.password_buf        = tmp;
-    opt.password_buf_size   = strlen(tmp) + 1;
+    opt.password_buf      = tmp;
+    opt.password_buf_size = strlen(tmp) + 1;
   }
 
   buf_avail_len = sizeof(buf) - UDP_HEADER_LEN;
@@ -795,7 +797,7 @@ static void *on_udp_write(oop_source *src, int sd,
   session = &slot->session;
 
   if (!session->must_send) {
-    if (session->udp_keepalive_send) {
+    if (session->keepalive_must_send) {
       nepim_udp_write_keepalive(slot, nepim_slot_buf_write,
                                 on_udp_keepalive_time, udp_slot_kill);
       return OOP_CONTINUE;
@@ -932,7 +934,7 @@ static void udp_slot_cancel_io(nepim_slot_t *slot)
     }
   }
   else {
-    if (session->udp_keepalive_send) {
+    if (session->keepalive_must_send) {
       if (slot->want_write) {
         assert(slot->want_write == 1);
         nepim_cancel_slot_keepalive(slot); /* keepalive write */
@@ -940,7 +942,7 @@ static void udp_slot_cancel_io(nepim_slot_t *slot)
       
       /* stop periodic keepalive scheduler */
       nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-                                        session->tv_keepalive,
+                                        session->tv_keepalive_send_next,
                                         on_udp_keepalive_time,
                                         slot);
     }
@@ -956,9 +958,9 @@ static void udp_slot_kill(nepim_slot_t *slot)
   udp_slot_cancel_stat_timers(slot);
   udp_slot_cancel_io(slot);
 
-  if (slot->session.udp_keepalive_require)
+  if (slot->session.keepalive_require)
     nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-                                      slot->session.tv_keepalive_timer,
+                                      slot->session.tv_keepalive_recv_timer,
                                       on_udp_keepalive_require, slot);
 
   cancel_slot_segment_read(slot);
@@ -1160,7 +1162,8 @@ static int spawn_udp_connection(const char *hostname,
                                      nepim_global.win_recv,
                                      nepim_global.win_send,
 				     0 /* tcp nodelay */,
-				     nepim_global.tos);
+				     nepim_global.tos,
+				     nepim_global.socket_keepalive);
     if (sd < 0) {
       if (ai->ai_family == AF_UNIX)
 	fprintf(stderr, 
@@ -1271,20 +1274,20 @@ static void parse_hosts(const char *host_list)
       /* add socket to table of readers/writers */
       nepim_usock_set_add(&udp_tab, sd);
 
-      opt.must_send             = nepim_global.duplex_mode
+      opt.must_send               = nepim_global.duplex_mode
         || nepim_global.simplex_client_send;
-      opt.bit_rate              = nepim_global.bit_rate;
-      opt.pkt_rate              = nepim_global.pkt_rate;
-      opt.stat_interval         = nepim_global.stat_interval;
-      opt.test_duration         = nepim_global.test_duration;
-      opt.write_delay           = nepim_global.write_delay;
-      opt.udp_keepalive_send    = nepim_global.udp_keepalive_send;
-      opt.udp_keepalive_require = nepim_global.udp_keepalive_require;
-      opt.seed                  = nepim_global.seed;
-      opt.verify_data           = nepim_global.verify_data;
-      opt.random_fill           = nepim_global.random_fill;
-      opt.fill_byte             = nepim_global.fill_byte;
-      opt.pause_duration        = nepim_global.pause_duration;
+      opt.bit_rate                = nepim_global.bit_rate;
+      opt.pkt_rate                = nepim_global.pkt_rate;
+      opt.stat_interval           = nepim_global.stat_interval;
+      opt.test_duration           = nepim_global.test_duration;
+      opt.write_delay             = nepim_global.write_delay;
+      opt.keepalive_must_send     = nepim_global.udp_keepalive_must_send;
+      opt.keepalive_require       = nepim_global.udp_keepalive_require;
+      opt.seed                    = nepim_global.seed;
+      opt.verify_data             = nepim_global.verify_data;
+      opt.random_fill             = nepim_global.random_fill;
+      opt.fill_byte               = nepim_global.fill_byte;
+      opt.pause_duration          = nepim_global.pause_duration;
       
       /* create UDP slots */
       for (i = 0; i < nepim_global.pipes; ++i) {
