@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: udp_server.c,v 1.75 2006/03/30 23:36:49 evertonm Exp $ */
+/* $Id: udp_server.c,v 1.80 2006/04/18 12:10:15 evertonm Exp $ */
 
 
 #include <stdio.h>
@@ -193,7 +193,7 @@ static void udp_slot_cancel_writers(nepim_slot_t *slot)
     }
   }
   else {
-    if (session->udp_keepalive_send) {
+    if (session->keepalive_must_send) {
       if (slot->want_write) {
 	assert(slot->want_write == 1);
 	nepim_cancel_slot_keepalive(slot); /* keepalive write */
@@ -201,7 +201,7 @@ static void udp_slot_cancel_writers(nepim_slot_t *slot)
 
       /* stop periodic keepalive scheduler */
       nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-					session->tv_keepalive,
+					session->tv_keepalive_send_next,
 					on_udp_keepalive_time, slot);
     }
   }
@@ -221,9 +221,9 @@ static void udp_slot_kill(nepim_slot_t *slot)
   udp_slot_cancel_stat_timers(slot);
   udp_slot_cancel_io(slot);
 
-  if (slot->session.udp_keepalive_require)
+  if (slot->session.keepalive_require)
     nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-				      slot->session.tv_keepalive_timer,
+				      slot->session.tv_keepalive_recv_timer,
 				      on_udp_keepalive_require, slot);
 
   nepim_sock_show_opt(stderr, slot->udp_sd);
@@ -532,7 +532,7 @@ static void *on_udp_write(oop_source *src, int sd,
   session = &slot->session;
 
   if (!session->must_send) {
-    if (session->udp_keepalive_send) {
+    if (session->keepalive_must_send) {
       nepim_udp_write_keepalive(slot, nepim_slot_buf_sendto,
 				on_udp_keepalive_time, udp_slot_kill);
       return OOP_CONTINUE;
@@ -554,16 +554,17 @@ static void *on_udp_keepalive_time(oop_source *src, struct timeval tv, void *use
   nepim_slot_t *slot = user;
   nepim_session_t *session = &slot->session;
 
-  assert(timercmp(&tv, &session->tv_keepalive, ==));
+  assert(timercmp(&tv, &session->tv_keepalive_send_next, ==));
   assert(!session->must_send);
-  assert(session->udp_keepalive_send);
+  assert(session->keepalive_must_send);
 
   /* save next scheduling time */
   {
-    int result = gettimeofday(&session->tv_keepalive, 0);
+    int result = gettimeofday(&session->tv_keepalive_send_next, 0);
     assert(!result);
   }
-  nepim_timer_usec_add(&session->tv_keepalive, nepim_global.udp_keepalive_delay);
+  nepim_timer_usec_add(&session->tv_keepalive_send_next,
+		       nepim_global.udp_keepalive_send_delay);
 
   /* start to write */
   nepim_will_slot_keepalive(slot, on_udp_write);
@@ -576,10 +577,10 @@ static void *on_udp_keepalive_require(oop_source *src, struct timeval tv, void *
   nepim_slot_t *slot = user;
   nepim_session_t *session = &slot->session;
 
-  assert(session->udp_keepalive_require);
-  assert(slot->keepalives_recv >= 0);
+  assert(session->keepalive_require);
+  assert(session->keepalives_recv >= 0);
 
-  if (slot->keepalives_recv < 1) {
+  if (session->keepalives_recv < 1) {
     fprintf(stderr, 
 	    "%d %d-%d: broken slot: peer not sending keepalives?\n",
 	    slot->udp_sd, slot->index, slot->index_remote);
@@ -591,7 +592,7 @@ static void *on_udp_keepalive_require(oop_source *src, struct timeval tv, void *
     return OOP_CONTINUE;
   }
 
-  slot->keepalives_recv = 0;
+  session->keepalives_recv = 0;
 
   nepim_schedule_keepalive_timer(slot, on_udp_keepalive_require);
 
@@ -620,16 +621,16 @@ static void udp_slot_start(int local_slot)
     }
   }
   else {
-    if (session->udp_keepalive_send) {
+    if (session->keepalive_must_send) {
 
       fprintf(stderr, 
 	      "DEBUG %s %s: %d %d-%d: scheduling sending of keepalives\n",
 	      __FILE__, __PRETTY_FUNCTION__,
 	      slot->udp_sd, slot->index, slot->index_remote);
       
-      session->tv_keepalive = OOP_TIME_NOW;
+      session->tv_keepalive_send_next = OOP_TIME_NOW;
       nepim_global.oop_src->on_time(nepim_global.oop_src,
-				    session->tv_keepalive,
+				    session->tv_keepalive_send_next,
 				    on_udp_keepalive_time, slot);
     }
   }
@@ -648,7 +649,7 @@ static void udp_slot_start(int local_slot)
 
   schedule_stat_interval(local_slot);
 
-  if (session->udp_keepalive_require)
+  if (session->keepalive_require)
     nepim_schedule_keepalive_timer(slot, on_udp_keepalive_require);
 }
 
@@ -745,17 +746,17 @@ static int parse_greetings(nepim_greet_t *opt,
 	  "%d %d-%d: UDP from (%s,%d): send=%d bps=%lld pps=%d "
 	  "interval=%d duration=%d delay=%ld ka_send=%d ka_req=%d seed=0x%08x "
 	  "verify_data=%d random_fill=%d fill_byte=0x%02x pause_duration=%d "
-	  "password=%s\n", 
+	  "sock_ka=%d password=%s\n", 
 	  sd, 
 	  local_slot, remote_slot,
 	  peer,
 	  nepim_sock_get_port((struct sockaddr *) remote),
 	  opt->must_send, opt->bit_rate, opt->pkt_rate,
 	  opt->stat_interval, opt->test_duration, 
-	  opt->write_delay, opt->udp_keepalive_send,
-	  opt->udp_keepalive_require, opt->seed,
+	  opt->write_delay, opt->keepalive_must_send,
+	  opt->keepalive_require, opt->seed,
 	  opt->verify_data, opt->random_fill, opt->fill_byte,
-	  opt->pause_duration,
+	  opt->pause_duration, opt->socket_keepalive,
 	  password_buf);
 
   if (nepim_global.tcpwrap) {
@@ -786,7 +787,7 @@ static void udp_consume(nepim_slot_t *slot, size_t len, uint32_t seq)
   nepim_session_t *session = &slot->session;
 
   /* any valid traffic accounted as keepalive */
-  ++slot->keepalives_recv;
+  ++session->keepalives_recv;
 
   nepim_session_read_add(session, len);
 
@@ -923,6 +924,12 @@ static void *on_udp_segment(oop_source *src, int sd,
 
     assert(local_slot < 0xFFFF);
 
+    /*
+      multiple UDP clients (slots) on the same socket will
+      probably clash distinct requests for socket keepalive
+     */
+    nepim_socket_keepalive(sd, opt.socket_keepalive);
+
     nepim_slot_set_add(&slots, sd, 
 		       local_slot, hdr.src_slot,
 		       (const struct sockaddr *) &from, 
